/*
 * Copyright 2001 Sun Microsystems, Inc.
 * Property of Sun Microsystems, Inc.
 */


package com.sun.im.provider.sample;

import org.netbeans.lib.collab.util.*;
import java.io.*;
import java.util.*;
import com.sun.im.provider.*;
import com.sun.im.service.*;


public class ExampleArchiveProvider extends ArchiveProvider implements PollArchiveProvider, NewsArchiveProvider
{
    private String _msgLogUrl; 
    private PrintWriter _pw;
    private java.text.SimpleDateFormat _dateFormat;

    private static final int JOIN = 1;
    private static final int CLOSE = 2;
    private static final int LEFT = 3;
    private static final int OPEN = 4;
    private static final int MESSAGE = 5;
    private static final int LOGIN = 6;
    private static final int LOGOUT = 7;

    public final static String ARCHIVE_URL = "iim_server.msg_archive.url";    
    ConfigReader config = ServerUtility.getInstance().getConfigReader();

    public ExampleArchiveProvider()
    {
	_msgLogUrl = config.getConfigValue(ARCHIVE_URL,
				   "/var/tmp/im_archive.log");
	_dateFormat = new java.text.SimpleDateFormat("yyyy.MM.dd HH.mm.ss.SSSS");
    }


    /** 
     * invoked when a user creates a private conference
     * @param conferenceAddress address of the conference
     * @param initiatorAddress address of initiator the conference
     */
    public void onSetup(String conferenceAddress, String initiatorAddress)
    {
	writeConferenceEvent(conferenceAddress, OPEN, initiatorAddress);
    }


    /** 
     * invoked when a user joins a conference
     * @param conferenceAddress address of the conference
     * @param participantAddress address of participant
     */
    public void onJoin(String conferenceAddress, String participantAddress)
    {
	writeConferenceEvent(conferenceAddress, JOIN, participantAddress);
    }


    /** 
     * invoked when a user leaves conference
     * @param conferenceAddress address of the conference
     * @param participantAddress address of participant
     */
    public void onLeave(String conferenceAddress, String participantAddress)
    {
	writeConferenceEvent(conferenceAddress, LEFT, participantAddress);
    }

    /** 
     * invoked when a private conference is terminated
     * @param conferenceAddress address of the conference
     * @param initiatorAddress address of initiator the conference
     */
    public void onClose(String conferenceAddress, String initiatorAddress)
    {
	writeConferenceEvent(conferenceAddress, CLOSE, initiatorAddress);
    }


    /** 
     * invoked when a user creates a private conference
     * @param conferenceAddress address of the conference
     * @param originatorAddress address of the invitor
     * @param recipientAddress addresses of the recipients of
     * this invitation.
     * @param message invite message
     */
    public void onInvite(String conferenceAddress,
			 ReadOnlyMessage message)
    {
	writeMessage(message);
    }


    /** 
     * invoked when a user posts a message to a conference
     * @param conferenceAddress address of the conference
     * @param initiatorAddress address of initiator the conference
     */
    public void onConferenceMessage(String conferenceAddress, 
				    ReadOnlyMessage message)
    {
	Date date = writeConferenceEvent(conferenceAddress, MESSAGE, 
					 message.getOriginator());
	writeMessage(message, date);
    }

    /** 
     * invoked whenever a non-conference message is received.
     * @param conferenceAddress address of the conference
     * @param initiatorAddress address of initiator the conference
     */
    public void onMessage(ReadOnlyMessage message)
    {
	writeMessage(message);
    }

    public void onMessage(java.util.List accessList,
				   com.sun.im.service.ReadOnlyMessage message)
    {
	onMessage(message);
    }


    private void writeMessage(ReadOnlyMessage message)
    {
	Date date = new Date();
	writeMessage(message, date);
    }

    private void writeMessage(ReadOnlyMessage message, Date date)
    {
	PrintWriter os = null;
	try {	    	    
	    os = getWriter();
	    if (os == null) return;
	    
	    StringBuffer ret = new StringBuffer("From: ");
	    ret.append(message.getOriginator());
	    ret.append("\r\nTo: ");
	    String[] to = message.getRecipients();
	    for (int i = 0; i < to.length; i++) {            
		ret.append(to[i]);
	    }
	    ret.append("\r\nMessage-Id: ");
	    ret.append(message.getMessageId());
	    ret.append("\r\nSubject: ");        
	    ret.append(message.getHeader("subject"));
	    ret.append("\r\nDate: ");
	    ret.append(date.toString());
	    ret.append("\r\n\r\n");

	    ReadOnlyMessagePart[] parts = message.getReadOnlyParts();
	    for (int i = 0 ; i < parts.length ; i++) {
		ret.append("Content-type: ");
		ret.append(parts[i].getContentType());
		ret.append("\r\n");
		try {
		    InputStream is = parts[i].getInputStream();
		    if (is != null) {
			BufferedReader br=new BufferedReader(new InputStreamReader(is));
			String str;
			while((str=br.readLine()) != null) ret.append(str);
		    }
		} catch (Exception e) { 
		    e.printStackTrace();
		}
	    }
	    
	    ret.append("\r\n.\r\n\r\n");

	    os.print(ret.toString());
	    os.flush();
	    
	} catch (Exception e) {
            e.printStackTrace();
	    closeWriter();
	}
    }
    
    private Date writeConferenceEvent(String conferenceAddress,
				      int type,
				      String uid)
    {
	Date date = new Date();
	PrintWriter os = null;
	try {	    	    
	    os = getWriter();
	    if (os == null) return date;
	    
	    StringBuffer ret = new StringBuffer();
	    ret.append(_dateFormat.format(date));
	    switch (type) {
	    case JOIN : 
		ret.append(" JOIN(");
		break;
	    case LEFT : 
		ret.append(" LEFT(");
		break;
	    case CLOSE : 
		ret.append(" CLOSE(");
		break;
	    case OPEN : 
		ret.append(" OPEN(");
		break;
	    case MESSAGE : 
		ret.append(" MESSAGE(");
		break;
	    case LOGIN : 
		ret.append(" LOGIN(");
		break;
	    case LOGOUT : 
		ret.append(" LOGOUT(");
		break;
	    default :
		ret.append(" ????(");
	    }

	    if (uid != null) {
		ret.append(uid);
		try {
		    //String dn = NMS.getDistinguishedName(uid);
		    String dn = "dn";
		    ret.append(" <");
		    ret.append(dn);
		    ret.append(">");
		} catch (Exception e) {
		    e.printStackTrace();
		}
	    }
	    
	    AccessControlList acl = null;
	    if (conferenceAddress != null) {
		ret.append(",");
		ret.append(conferenceAddress);
		//acl = NMS.getAccessControlList(conferenceAddress);
	    }
	    ret.append(")");

	    if (acl != null) {
		ret.append("\r\n Access list \ndefault access: ");
		ret.append(acl.getDefaultAccess());
		ret.append("\r\n");
		for (Enumeration e = acl.getAccessControlItems();
		     e.hasMoreElements(); ) {
		    AccessControlItem aci = (AccessControlItem)e.nextElement();
		    ret.append("    " + aci.subjectUID + " <" + aci.subjectDistinguishedName + ">: " + aci.access + "\r\n");		
		}
	    }
	    ret.append("\r\n.\r\n");

	    os.print(ret.toString());
	    os.flush();
	    
	} catch (Exception e) {
	    e.printStackTrace();
	    closeWriter();
	}
	return date;
    }

    private synchronized PrintWriter getWriter() 
    {
	if (_pw == null) {
	    try {	    
		_pw = new PrintWriter(new FileWriter(_msgLogUrl, true));
	    } catch (Exception e) {
		e.printStackTrace();
	    }
	}
	return _pw;
    }

    private synchronized void closeWriter()
    {
	try {
	    if (_pw != null) {
		_pw.close();
	    }
	} catch (Exception e) { }
	_pw = null;
    }

    /**
     * open the archive
     * @exception Exception failure to open and initialize the 
     * archive.
     */
    public void open() throws Exception
    {
	//make sure the directory exists
	File f = new File(_msgLogUrl);
	File dir = f.getParentFile();
	if (dir != null && !dir.exists()) dir.mkdirs();
    }


    /** 
     * invoked when a user signs on
     * @param uid identifier of the authenticated user
     */
    public void onLogin(String uid)
    {
	writeConferenceEvent(null, LOGIN, uid);
    }

    /** 
     * invoked when a user signs off
     * @param uid user identifier
     */
    public void onLogout(String uid)
    {
	writeConferenceEvent(null, LOGOUT, uid);
    }


    /**
     * invoked when a new poll message is received
     * @param originator poll answer sender
     * @param pollData poll data
     */
    public void newPoll(String originator,
			com.sun.im.service.Poll pollData)
    {
	try {
	    PrintWriter os = getWriter();
	    StringBuffer ret = new StringBuffer();
	    ret.append(" poll\n");
	    ret.append(pollData.toString());
	    os.print(ret.toString());
	    os.flush();	    
	} catch (Exception e) {
	    e.printStackTrace();
	    closeWriter();
	}
    }

    /**
     * invoked when a poll answer to an existing poll is received.
     * @param originator poll answer sender
     * @param pollId unique poll ID
     * @param answer answer
     */
    public void pollAnswer(String originator, String pollId, String answer)
    {
	try {
	    PrintWriter os = getWriter();;
	    StringBuffer ret = new StringBuffer();
	    ret.append(" poll reply\n");
	    os.print(ret.toString());
	    os.flush();	    
	} catch (Exception e) {
	    e.printStackTrace();
	    closeWriter();
	}
    }

    /**
     * invoked when a new news message is received
     * @param originator news message creator
     * @param message news message
     */
    public void onNewsMessage(String addr, ReadOnlyMessage message)
    {
	PrintWriter os = getWriter();;
	os.println("News Message Received");
	writeMessage(message);
    }

    
    /** 
     * invoked when a user joins a news channel
     * @param addr address of the news channel
     * @param uid unique identifier of the new subscriber
     */
    public void onSubscribe(String addr, String uid)
    {
    }

    /** 
     * invoked when a user leaves news channel
     * @param addr address of the news channel
     * @param uid unique identifier of the leaving subscriber
     */
    public void onUnsubscribe(String addr, String uid)
    {
    }



}
